#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a drawing look  

# (c) Dinasset 2021
#
#   History:
#
#   v1.0: 2021-02-28: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

 
from gimpfu import *
import random
from random import randint
import sys, os
import time
import datetime

# define here some constant value, when required
FixWH=3000

############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

####################
#   M A I N
#-------------------
    
def DrawStyleT (inImage, inDrawable, outimgWH, preproc,
                    look, postProc, withBorders, Mask,
                    doFlatten) :
    
    pdb.gimp_context_push()
    currentFG = pdb.gimp_context_get_foreground()
    # source image characteristics
    width = inImage.width
    height = inImage.height
    OrigWH = width+height
    otype = inImage.base_type
    # image name
    iname = inImage.name
    split = iname.split("(", 1)
    iname = split[0]
    end = len(iname)-2
    iname = iname[1:end]    # remove []
    #info_msg (iname)
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H-%M-%S')
    
    Ratio = FixWH*1.00/OrigWH*1.00
    rWidth = int(width*Ratio)
    rHeight = int(height*Ratio)
    WorkWH = rWidth+rHeight
    
    # START    
    # create output image
    newImage = pdb.gimp_image_new (rWidth, rHeight, otype )
    newBG = pdb.gimp_layer_new_from_drawable (inDrawable, newImage)
    newImage.add_layer (newBG, 0)
    pdb.gimp_layer_scale (newBG, rWidth, rHeight, False)
    newBG.name = inDrawable.name
    if '.' in newBG.name:
        split = newBG.name.split(".", 1)
        newBG.name = split[0]
    # ofnuts technique to give it a name (unsaved). I set it to the name of the original layer
    newImage.filename=os.path.join(gimp.user_directory(4),iname)
    
    #pdb.gimp_display_new (newImage)
    #return()
    
    if preproc==True:
        # adjust source values
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG, HISTOGRAM_VALUE,
                                                    0.0, 1.0)
        gimp.message ("mean="+str(mean))
        if mean<95.5:
            while mean<95.5:
                pdb.gimp_drawable_levels (newBG, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.10, 0.0, 1.00, False)
                (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG, HISTOGRAM_VALUE,
                                                            0.0, 1.0)
        elif mean>160.0:
            while mean>160.0:
                pdb.gimp_drawable_levels (newBG, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.91, 0.0, 1.00, False)
                (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG, HISTOGRAM_VALUE,
                                                            0.0, 1.0)
        # remove blacks and whites     
        pdb.gimp_drawable_curves_spline (newBG, HISTOGRAM_VALUE,  
                                     6, (0.0,0.1, 0.5,0.5, 1.0, 0.9))
                                     
    # Change colour to a light pastel nuance
    # - create the border with DoG
    normalized_layer=newBG.copy()
    newImage.add_layer (normalized_layer, 0)
    normalized_layer.name="Normalized"
    pdb.gimp_drawable_curves_spline (normalized_layer, HISTOGRAM_VALUE, 6,
                                     ( 0., 0.2, 0.5, 0.625, 1.0, 1.0)) 
    pdb.gimp_selection_none(newImage)
    pdb.gimp_drawable_hue_saturation(normalized_layer, HUE_RANGE_ALL, 0.0, 0.0, 100.0, 0.0)
    #pdb.gimp_palette_delete(temp_palette)
    
    #pdb.gimp_display_new (newImage)
    #return()
     
    # Third part: create the border with DoG
    #
    if withBorders==True:
        borders_layer=normalized_layer.copy()
        newImage.add_layer (borders_layer, 0)
        borders_layer.name="Borders"
        borders_layer.mode=LAYER_MODE_GRAIN_MERGE
        borders_layer.opacity=66.0
        Radius=5.0
        Amount=0.25 # was 0.1
        pdb.plug_in_neon (newImage, borders_layer,
                          Radius, Amount)
        #pdb.gimp_drawable_curves_spline (borders_layer, 0, 6,
        #                        (0.0,0.0, 0.2,0.0, 1.0,1.0))  
        pdb.plug_in_vinvert (newImage, borders_layer)
        #pdb.gimp_drawable_curves_spline (borders_layer, 0, 6,
        #                        (0.0,0.4, 0.8,0.6, 1.0,1.0))  
        pdb.gimp_layer_add_alpha (borders_layer)
     
    finalPoster=pdb.gimp_layer_new_from_visible(newImage,newImage,
                                                "final poster ")
    newImage.add_layer(finalPoster,0)
    finalPoster.mode=LAYER_MODE_NORMAL
    
    #pdb.gimp_display_new (newImage)
    #return()
     
# =======================================================================================
        
    pdb.plug_in_gmic_qt(newImage, finalPoster, 1,2 ,
    #[G'MIC] Hard Sketch: fx_hardsketchbw 1580,   14,  6.59,0.291,5,0,4,0,50,50
    #"-v - -fx_hardsketchbw 1580,14,6.6,0.29,5,0,4,0,50,50 ")
    #[G'MIC] Hard Sketch: fx_hardsketchbw 300,50,1,0.1,20,0,4,0,50,50 (std)
    "-v - -fx_hardsketchbw 600,30,1,0.15,20,0,4,0,50,50 ")  # was 0.1
    sketched_layer = newImage.layers[0]
    sketched_layer.name = "Hard sketched"
    sketched_layer.mode = LAYER_MODE_HSV_VALUE
    sketched_layer.opacity = 70.0
    pdb.plug_in_spread (newImage, sketched_layer, 1.0, 1.0)
    
    #pdb.gimp_display_new (newImage)
    #return()
     
    strokeMode = "sketched, "
    PreOutcome = pdb.gimp_layer_new_from_visible(newImage,newImage,
                                          " preliminary outcome")
    newImage.add_layer(PreOutcome,0)
    PreOutcome.mode=LAYER_MODE_NORMAL
    PreOutcome.opacity = 100.0
    if look == 1:
        pdb.gimp_drawable_desaturate (PreOutcome, DESATURATE_LUMA)
    elif look == 2:
        PreOutBW = PreOutcome.copy()
        newImage.add_layer(PreOutBW,0)
        pdb.gimp_drawable_desaturate (PreOutBW, DESATURATE_LUMA)
        PreOutBW.opacity = 50.0
        PreOutcome = pdb.gimp_image_merge_down (newImage, PreOutBW,
                                                CLIP_TO_BOTTOM_LAYER)
    
    BGonTop = newBG.copy()
    newImage.add_layer(BGonTop,0)
    BGonTop.name = "BG on Top "
    BGonTop.mode = LAYER_MODE_LINEAR_LIGHT
    #graymask = pdb.gimp_layer_create_mask (BGonTop, ADD_MASK_COPY)
    BGonTop.opacity = 33.0  # when a mask is not applied 
    if Mask>0:
        BGonTop.opacity = 90.0  # when a mask is applied                                                                    
        if Mask==1:
            blackmask = pdb.gimp_layer_create_mask (BGonTop, ADD_MASK_BLACK)    
            pdb.gimp_layer_add_mask (BGonTop, blackmask)
            optmask=", black mask"
        if Mask in (2,3):
            graymask = pdb.gimp_layer_create_mask (BGonTop, ADD_MASK_COPY)      
            pdb.gimp_layer_add_mask (BGonTop, graymask)
            if Mask==2: 
                optmask=", gray-copy mask"
            if Mask==3:
                pdb.plug_in_vinvert (newImage, graymask)
                optmask=", gray-copy inverted mask"
        if Mask==4:
            midgraymask = pdb.gimp_layer_create_mask (BGonTop, ADD_MASK_BLACK)    
            pdb.gimp_layer_add_mask (BGonTop, midgraymask)
            pdb.gimp_drawable_curves_spline (midgraymask, HISTOGRAM_VALUE,
                                             4, (0.0,0.5, 1.0,0.5))
            optmask=", mid gray mask"
    else:   optmask=", basic"
    #pdb.gimp_layer_add_mask (BGonTop, graymask)
    #pdb.plug_in_vinvert (newImage, graymask)
    
    if look in (1,2):
        pdb.gimp_drawable_desaturate (BGonTop, DESATURATE_LUMA)
        if look==1: BGonTop.opacity = 95.0
        else: BGonTop.opacity = 45.0
        
    if postProc>0:  # 0=*none
        forPostProcess = pdb.gimp_layer_new_from_visible(newImage,newImage, "Post-process")
        newImage.add_layer(forPostProcess, 0)
        if postProc==1:  # 1=apply curves
            pdb.gimp_drawable_curves_spline (forPostProcess, 0, 8,
                              (0.0,0.0, 0.2,0.1, 0.5,0.3, 0.8,0.9, 1.0,1.0))
            forPostProcess.mode = LAYER_MODE_NORMAL
            forPostProcess.opacity = 100.0
        if postProc==2:  # 2=apply linear light
            forPostProcess.mode = LAYER_MODE_MULTIPLY
            forPostProcess.opacity = 75.0
        if postProc==3:  # 2=apply linear burn
            forPostProcess.mode = LAYER_MODE_LINEAR_BURN
            forPostProcess.opacity = 65.0
    
    
    Outcome = pdb.gimp_layer_new_from_visible(newImage,newImage, "")
    newImage.add_layer(Outcome,0)
    
    if doFlatten==True:
        pdb.gimp_item_set_visible (newBG, False) # temp
        Outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_IMAGE)
        pdb.gimp_item_set_visible (newBG, True) # reset
    else:
        pdb.gimp_item_set_visible (Outcome, False)
        
    looks=["coloured","grayscale","mix"]
    strokeMode=strokeMode+looks[look]
    Outcome.name = st+" "+newBG.name+" stroked "+strokeMode
    
    if outimgWH==0: outimgWH=OrigWH
    uRatio = outimgWH*1.00/WorkWH*1.00
    uWidth = int(rWidth*uRatio)
    uHeight = int(rHeight*uRatio)
    #info_msg ("SCALING")   # not during tests, scaling is slow
    pdb.gimp_image_scale (newImage, uWidth, uHeight)   # to user chosen size
        
    pdb.gimp_display_new(newImage)
    
    # tier down
    
    pdb.gimp_selection_none(newImage)
    pdb.gimp_context_pop()
    
    #pdb.gimp_display_new(newImage)
     
    return(Outcome)

# This is the plugin registration function

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset (inspired by Lyle)"
menu="<Image>/Diego/Drawing/"
copyrightYear="2021"
desc="Draw Effect Style T "

register(
    "DrawStyleT",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+"...",
    "*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of New Image"+"\n (0=same as input)",+
                              3500, (0, 8000, 100)),
      (PF_TOGGLE, "preproc", "Pre-process Image?", False),
      (PF_OPTION, "look", "Look of outcome", 0, ["Coloured",
                                                 "B&W",
                                                 "Mix",
                                                  ]),
      (PF_OPTION, "postpro", "Post-process touch-up?"+
                            "\n  (for colours/contrast adjustments)", 0, ["*none",
                                                 "curves","multiply","linear burn"]),
      (PF_TOGGLE, "borders", "Show borders?", True),
      (PF_OPTION, "maskbg", "Add Masked to BG on Top?"+
                            "\n  (for adjustments)", 0, ["*none (basic)",
                                                 "black (user paints over)",
                                                 "gray copy",
                                                 "gray copy inverted",
                                                 "flat medium gray",
                                                ]),
      (PF_TOGGLE, "flatten", "Flatten image?", False),
    ], 
    [
      (PF_DRAWABLE, "drawable", "Output drawable", None),
    ],
    DrawStyleT,
    menu=menu
    )

main()
